<?php

namespace Drupal\wvn_api\Services;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\wvn_api\Entity\Job;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * The sync service.
 *
 * @package Drupal\wvn_api\Services
 */
class WvnSync {
  use StringTranslationTrait;

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * API endpoint.
   *
   * @var string
   */
  private $endpoint;

  /**
   * API username.
   *
   * @var string
   */
  private $username;

  /**
   * API password.
   *
   * @var string
   * @todo Ignore password in config export.
   */
  private $password;

  /**
   * API organisation selector.
   *
   * @var string
   */
  private $selector;

  /**
   * Allowed status.
   *
   * Only process jobs with this status.
   *
   * @var array
   */
  private $allowedStatus = [];

  /**
   * WvnSync constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, ConfigFactoryInterface $config_factory) {
    $this->entityTypeManager = $entity_type_manager;
    $config = $config_factory->get('wvn_api.settings');
    $this->endpoint = $config->get('endpoint');
    $this->allowedStatus = $config->get('allowed_status');
    $this->username = $config->get('username');
    $this->password = $config->get('password');
    $this->selector = $config->get('selector');
  }

  /**
   * Gets API key.
   *
   * Fetches a API key from the webservice for authentication.
   */
  private function getApiKey() {
    $postData = [
      'username' => $this->username,
      'password' => $this->password,
    ];

    $return = $this->executeRequest('getApiKey.json', $postData);

    return $return['result'];
  }

  /**
   * Executes request.
   *
   * Execute action for API.
   *
   * @param string $action
   *   API action.
   * @param array $postData
   *   Post data.
   *
   * @return array
   *   API response data.
   */
  private function executeRequest($action, array $postData) {

    $url = $this->endpoint . $action;

    $curlHandler = curl_init($url);

    curl_setopt_array($curlHandler, [
      CURLOPT_POST => TRUE,
      CURLOPT_RETURNTRANSFER => TRUE,
      CURLOPT_HTTPHEADER => ['Content-Type: application/json'],
      CURLOPT_POSTFIELDS => json_encode($postData),
    ]);

    $response = curl_exec($curlHandler);

    if ($response === FALSE) {
      throw new \Exception(curl_error($curlHandler));
    }

    $responseData = json_decode($response, TRUE);

    return $responseData;

  }

  /**
   * Updates jobs.
   *
   * @return int
   *   The amount of jobs processed.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function updateJobs() {
    // @todo Don't delete jobs on every Sync.
    $this->deleteJobs();

    $processed = 0;
    foreach ($this->getJobs() as $job) {
      if ($this->processJobs($job)) {
        $processed++;
      }
    }
    return $processed;
  }

  /**
   * Gets jobs.
   *
   * Fetch jobs from API.
   */
  public function getJobs() {
    $apiKey = $this->getApiKey();

    $postData = [
      'apiKey' => $apiKey,
      'filter' => [
        '__type__' => 'RemoteJobFilter',
      ],
      'fieldSelection' => [
        '__type__' => 'RemoteJobFieldselection',
        'jobContent' => [
          '__type__' => 'RemoteJobContentFieldSelection',
          'department' => TRUE,
          'employer' => TRUE,
        ],
        'organisation' => TRUE,
        'jobFeatures' => [
          '__type__' => 'RemoteJobFeaturesFieldSelection',
          'applicationContact' => TRUE,
          'applicationProcedureContact' => TRUE,
          'applicationTypes' => TRUE,
          'detail' => TRUE,
          'employmentConditions' => TRUE,
          'informationContact' => TRUE,
          'location' => TRUE,
        ],
        'jobPublication' => [
          '__type__' => 'RemoteJobPublicationDataFieldSelection',
          'publicationSets' => TRUE,
        ],
      ],
    ];

    $returnData = $this->executeRequest('getJobs.json', $postData);
    $return = [];

    if (!empty($this->selector)) {
      foreach ($returnData['result'] as $job) {
        if (strstr($job['features']['internalReferenceCode'], $this->selector)) {
          $return[] = $job;
        }
      }
    }
    else {
      return $returnData['result'];
    }
    return $return;
  }

  /**
   * Processes jobs from API.
   *
   * @param array $job
   *   The job to process.
   *
   * @return bool
   *   True if entity is successfully saved, false otherwise.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function processJobs(array $job) {
    if ($this->checkPublished($job['publication'])) {
      $exists = $this->checkIfExists($job);

      // @todo Only update entity if there are changes to be imported.
      if ($exists) {
        reset($exists);
        $key = key($exists);
        /** @var \Drupal\wvn_api\Entity\Job $entity */
        $entity = $this->entityTypeManager->getStorage('wvn_api_job')->load($exists[$key]);
      }
      else {
        $entity = Job::create(['type' => 'wvn_api_job']);
      }

      // @todo Check if there is more information to save.
      $entity->set('name', $job['content']['name']);
      $entity->set('wvn_id', trim($job['externalReferenceCode']));
      $entity->set('description', $job['content']['description']);
      $entity->description->setValue([
        'value' => $job['content']['description'],
        'format' => 'full_html',
      ]);
      $entity->set('location', $job['features']['location']['city']);
      // Create uri like https://www.werkenvoornederland.nl/LOG180048
      $entity->link = [
        'uri' => 'https://www.werkenvoornederland.nl/' . $job['externalReferenceCode'],
        'title' => $this->t('Read more'),
      ];

      if ($entity->save()) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Checks if vacancy is published.
   *
   * Only enable external (Extern) and published jobs (between start and
   * end date).
   *
   * @param array $publication
   *   The publication sets of the vacancy.
   *
   * @return bool
   *   TRUE if vacancy is published, FALSE otherwise.
   */
  protected function checkPublished(array $publication) {
    foreach ($publication['publicationSets'] as $publicationSet) {
      foreach ($publicationSet['publications'] as $publication) {
        $dateStart = $publication['dateStart'];
        $dateEnd = $publication['dateEnd'];
        if (in_array($publication['publicationStatus']['label'], $this->allowedStatus)) {
          // API returns date fields in microtime.
          $microtime = (int) round(microtime(TRUE) * 1000);
          if (empty($dateEnd)) {
            if ($dateStart < $microtime) {
              return TRUE;
            }
          }
          else {
            if ($dateStart < $microtime && $dateEnd > $microtime) {
              return TRUE;
            }
          }
        }
      }
    }
    return FALSE;
  }

  /**
   * Checks if item exists.
   *
   * @param array $job
   *   The job.
   *
   * @return array|bool
   *   The existing entity ids or false if none found.
   */
  protected function checkIfExists(array $job) {
    $entity_ids = $this->entityTypeManager->getStorage('wvn_api_job')->loadByProperties([
      'wvn_id' => trim($job['externalReferenceCode']),
    ]);

    return $entity_ids;
  }

  /**
   * Delete jobs.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function deleteJobs() {
    $ids = $this->entityTypeManager->getStorage('wvn_api_job')->loadMultiple();
    $this->entityTypeManager->getStorage('wvn_api_job')->delete($ids);
  }

}
